/* 
	MGAnimePanel.java

	Author:			Tomokazu ARITA
	since:			2001/12/29
*/

package hiformed.debug;

import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.geom.*;

import hiformed.grammar.*;
import hiformed.editor.mgpanel.*;


public class MGAnimePanel extends JPanel{
	
	//Graphics
	private BufferedImage buf;
	private Graphics2D bufg2d;	
	
	private static int WIDTH  = 500;
	private static int HEIGHT = 1300;
	
	//
	private MarkedGraph marked_graph;
	private MGNodeMarkSet node_set;
	
	//?
	//private static JFrame frame;
	//private static MarkedGraphPanel panel;
	
	
	
	private BlockManeger BM;//
	
	public MGAnimePanel(){
	}

	
	public Dimension getPreferredSize(){
		return new Dimension( WIDTH, HEIGHT);
	}
	
	
	public Dimension getMinimumSize(){
		return getPreferredSize();
	}

	
	int paint_flag = 0;
	public void paint(Graphics g){
		Graphics2D g2 = (Graphics2D)g;
		
		if(paint_flag == 0){
			initGraphics();
			if(marked_graph != null){ redrawMarkedGraph(); }//
			paint_flag++;
		}
		
		g2.drawImage(buf,0,0,this);
	}
	
	public void update(Graphics g){
		paint(g);
	}
	
	
	private void initGraphics(){
		Dimension dim = getSize();
		
		buf = (BufferedImage)createImage(dim.width, dim.height);
		bufg2d = (Graphics2D)buf.getGraphics();
		
		bufg2d.setColor(Color.white);
		bufg2d.fillRect(0, 0, dim.width, dim.height);
		
		/*
		RenderingHints qualityHints = 
				new RenderingHints(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);

		qualityHints.put(RenderingHints.KEY_RENDERING,
						RenderingHints.VALUE_RENDER_QUALITY);

		bufg2d.setRenderingHints(qualityHints);
		*/
	}
	
	private void setBlock(){
	
		Block.setMargin(Block.TOP,    10);
		Block.setMargin(Block.BOTTOM, 10);
		Block.setMargin(Block.LEFT,   10);
		Block.setMargin(Block.RIGHT,  10);
		Block.setMargin(Block.CENTER, 5, 5);
		
		Block.setCellMargin(Block.TOP,    5);
		Block.setCellMargin(Block.BOTTOM, 5);
		Block.setCellMargin(Block.LEFT,   5);
		Block.setCellMargin(Block.RIGHT,  5);
		
		Block.setNodeRadius(3);
	}
	
	private void drawBlock(Block block){
		Point point = new Point(50,50);
		
		
		bufg2d.setColor(Color.blue);
		Rectangle rect = new Rectangle();
		rect.setLocation(block.getCellPoint(point));
		rect.setSize(block.getCellWidth(bufg2d), block.getCellHeight(bufg2d));
		bufg2d.draw(rect);
		
		bufg2d.setColor(Color.green);
		Point node_point = block.getNodePoint(point,bufg2d);
		bufg2d.fillOval(node_point.x, node_point.y, 
						block.getNodeRadius()*2, block.getNodeRadius()*2);
		
		bufg2d.setColor(Color.gray);
		bufg2d.setFont(block.getTitleFont());
		Point string_point = block.getStringPoint(point,bufg2d);
		bufg2d.drawString(block.getTitle(), string_point.x, string_point.y);
						
	}
	
	
	private void drawBlacks(Block block){
		
		int x = 0;
		int y = 0;
		Block b1;
		
		
		for(int i=1; i<block.getXLocation(); i++){
			x += BM.BLOCKWIDTH[i];
		}
		
		for(int i=1; i<block.getYLocation(); i++){
			y += BM.BLOCKHEIGHT[i];
		}

		
		drawBlock2(block, x, y);
		
		if((b1 = block.getNeighborBlock(Block.IN,Block.END)) != null){
			drawEdge(block,b1,"in");
			drawBlacks(b1);
		}
		
		if((b1 = block.getNeighborBlock(Block.LF,Block.START)) != null){
			drawEdge(block,b1,"lf");
			drawBlacks(b1);
		}
		
		if((b1 = block.getNeighborBlock(Block.OV,Block.START)) != null){
			drawEdge(block,b1,"ov");
			drawBlacks(b1);
		}
	}
	
	
	private void drawBlock2(Block block,int x, int y){
		Point point = new Point(x,y);
		
		
		bufg2d.setColor(Color.black);
		Rectangle rect = new Rectangle();
		rect.setLocation(block.getCellPoint(point));
		rect.setSize(block.getCellWidth(bufg2d), block.getCellHeight(bufg2d));
		bufg2d.draw(rect);
		
				
		bufg2d.setColor(Color.black);
		Point node_point = block.getNodePoint(point,bufg2d);
		bufg2d.fillOval(node_point.x, node_point.y, 
						block.getNodeRadius()*2, block.getNodeRadius()*2);
		
		bufg2d.setColor(Color.gray);
		bufg2d.setFont(block.getTitleFont());
		Point string_point = block.getStringPoint(point,bufg2d);
		bufg2d.drawString(block.getTitle(), string_point.x, string_point.y);
		
	}
	
	private void drawEdge(Block block1, Block block2, String edge_label){
		
		int x1=0, x2=0, y1=0, y2=0;
	
		for(int i=1; i<block1.getXLocation(); i++){
			x1 += BM.BLOCKWIDTH[i];
		}
		
		for(int i=1; i<block1.getYLocation(); i++){
			y1 += BM.BLOCKHEIGHT[i];
		}
		
		for(int i=1; i<block2.getXLocation(); i++){
			x2 += BM.BLOCKWIDTH[i];
		}
		
		for(int i=1; i<block2.getYLocation(); i++){
			y2 += BM.BLOCKHEIGHT[i];
		}
		
		
		Point point1 = new Point(x1,y1);
		Point point2 = new Point(x2,y2);
		Point node_point1 = block1.getNodePoint(point1,bufg2d);
		Point node_point2 = block2.getNodePoint(point2,bufg2d);
		
		bufg2d.setColor(Color.black);
		bufg2d.drawLine(node_point1.x + block1.getNodeRadius(),
						 node_point1.y + block1.getNodeRadius(),
						 node_point2.x + block2.getNodeRadius(),
						 node_point2.y + block2.getNodeRadius());
		
		
		int string_point_x = ( node_point1.x + block1.getNodeRadius() 
								+ node_point2.x + block2.getNodeRadius() ) / 2
								- 12;
		
		int string_point_y = ( node_point1.y + block1.getNodeRadius()
								+ node_point2.y + block2.getNodeRadius() ) / 2
								+ 10;
		
		bufg2d.setColor(Color.gray);
		bufg2d.setFont(block1.getTitleFont());
		bufg2d.drawString(edge_label, string_point_x, string_point_y);
		
	}
	
	
	public Block makeBlocks(MarkedGraph graph){
		MGNode node = graph.getStartNode();
		//Block block = new Block(Integer.toString(node.getNodeLabel()));
		Block block = new Block(node_set.getNodeString(node.getNodeLabel()));
		subMakeBlocks(node, block);
		return block;
	}
	
	private Block subMakeBlocks(MGNode node, Block block){
		MGNode temp_node;
				
		if((temp_node = node.getInEndNode()) != null){
			//Block temp_block = new Block(Integer.toString(temp_node.getNodeLabel()));
			Block temp_block = new Block(node_set.getNodeString(temp_node.getNodeLabel()));
			block.setBlockPointer(Block.IN, Block.END,temp_block);
			temp_block.setBlockPointer(Block.IN, Block.START,block);
			subMakeBlocks(temp_node,temp_block);
		}
		
		if((temp_node = node.getOvStartNode()) != null){
			//Block temp_block = new Block(Integer.toString(temp_node.getNodeLabel()));
			Block temp_block = new Block(node_set.getNodeString(temp_node.getNodeLabel()));
			block.setBlockPointer(Block.OV, Block.START,temp_block);
			temp_block.setBlockPointer(Block.OV, Block.END,block);
			subMakeBlocks(temp_node,temp_block);
		}
		
		if((temp_node = node.getLfStartNode()) != null){
			//Block temp_block = new Block(Integer.toString(temp_node.getNodeLabel()));
			Block temp_block = new Block(node_set.getNodeString(temp_node.getNodeLabel()));
			block.setBlockPointer(Block.LF, Block.START,temp_block);
			temp_block.setBlockPointer(Block.LF, Block.END,block);
			subMakeBlocks(temp_node,temp_block);
		}
		
		return block;
	}
	
	
	//OtVȂ,͒lςƂKĂяo.
	public void setMarkedGraph(MarkedGraph graph){
		this.marked_graph = graph;
		paint_flag = 0;
		repaint();
	}
	
	private void redrawMarkedGraph(){
		setBlock();
		Block block = makeBlocks(marked_graph);
		
		BM = new BlockManeger(block);
		block = BM.getStartBlock(block);
		
		
		BM.initBlockList(block);
		BM.getWholeSize();
		BM.setLocation(block,1,1);
		BM.initSizeForDrawing(bufg2d);
		
		
		drawBlacks(block);
	}
	
	public void setNodeSet(MGNodeMarkSet node_set){
		this.node_set = node_set;
	}
}
