/* 
	Cell.java

	Author:	Tomokazu ARITA
	Date:   2001/02/28
	update : 2001/11/30
*/

package hiformed.editor.table;

import java.awt.Point;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Color;
import java.awt.Font;

public class Cell{
	
	//
	static public final boolean ON  = true;
	static public final boolean OFF = false;

	//CELL Size
	private int x = 0;
	private int y = 0;
	private int width  = 0;
	private int height = 0;
	
	private int top    = -1;
	private int bottom = -1;
	private int left   = -1;
	private int right  = -1;
	
	private Point   base_point    = new Point();//
	
	private Color   cell_bgcolor  = Color.white;
	private boolean cell_bg_onoff = ON;
	
	private boolean frame_line_onoff = ON;
	private int     flame_line_size  = 1;
	private Color   frame_line_color = Color.black;
	
	
	
	
	//Title
	private Point   title_base_point = new Point();
	//private int     title_font_size;
	//private int     title_font_style;
	//private String  title_font_name;
	private Font    title_font;
	private Color   title_color;
	private String  title_text;
	private boolean title_onoff = ON;
	
	
	
	//Construct 1.
	public Cell(){
		this.title_onoff = OFF;
	}
	
	//Construct 2.
	public Cell(String title){
		this.title_text       = title;
		
		String title_font_name  = "Helvatica";
		int    title_font_size  = 12;
		int    title_font_style = Font.BOLD;
		
		this.title_font       = new Font(title_font_name, title_font_style, title_font_size);
		this.title_color      = Color.black;
		this.title_onoff      = ON;
	}
	
	
	
	//set base point
	public void setBasePoint(int x, int y){
		this.x = x;
		this.y = y;
	}
	
	//get base point
	public Point getBasePoint(){
		return new Point(x,y);
	}
	
	
	//set cell size
	public void setCellSize(int width, int height){
		this.width  = width;
		this.height = height;
	}
	
	//get cell size
	public Dimension getCellSize(){
		return new Dimension(width, height);
	}
	
	
	
	//set side margen
	public void setSideMargin(int top, int left, int bottom, int right){
		this.top    = top;
		this.left   = left;
		this.bottom = bottom;
		this.right  = right;
	}
	
	//get side margen
	public Insets getSideMargin(){
		return new Insets(top, left, bottom, right);
	}
	
	
	
	//set cell background color
	public void setCellBGColor(Color bgcolor){
		this.cell_bgcolor = bgcolor;
	}
	
	//get cell background color
	public Color getCellBGColor(){
		return cell_bgcolor;
	}
	
	
	//set cell background color switch on or off
	public void setCellBGColorOnOff(boolean onoff){
		this.cell_bg_onoff = onoff;
	}
	
	//get cell background color switch
	public boolean getCellBGColorOnOff(){
		return cell_bg_onoff;
	}
	
	
	//set frame on or off
	public void setFrameOnOff(boolean onoff){
		this.frame_line_onoff = onoff;
	}
	
	//get frame on or off
	public boolean getFrameOnOff(){
		return frame_line_onoff;
	}
	
	
	//set frame_line_color on or off
	public void setFrameColor(Color frame_color){
		this.frame_line_color = frame_color;
	}
	
	//get frame_line_color on or off
	public Color getFrameColor(){
		return frame_line_color;
	}
	
	// set frame_line size
	public void setFrameLineSize(int size){
		this.flame_line_size = size;
	}
	
	// get frame_line size
	public int getFrameLineSize(){
		return flame_line_size;
	}
	
	
	//set title font
	public void setTitleFont(Font font){
		this.title_font = font;
	}
	
	//get title font
	public Font getTitleFont(){
		return title_font;
	}
	
	//set title color
	public void setTitleColor(Color color){
		this.title_color = color;
	}
	
	//get title color
	public Color getTitleColor(){
		return title_color;
	}
	
	//set title text
	public void setTitle(String title){
		this.title_text = title;
	}
	
	//get title text
	public String getTitle(){
		return title_text;
	}
	
	//set title on or off
	public void setTitleOnOff(boolean onoff){
		this.title_onoff = onoff;
	}
	
	//get title on or off
	public boolean getTitleOnOff(){
		return title_onoff;
	}
}